import pystan
from pystan import StanModel
import pickle
from multiprocessing.pool import Pool
import numpy as np
import matplotlib.pyplot as plt
from InputData import *
from SubModule import *
from CalcRho_cat import *
from RNGen import *


if __name__ == '__main__':

    #
    #       Prepare the input data file
    #

    fn_in = input("Input data file = ")
    f_in = open(fn_in, "r")

    #
    #       Prepare the output file
    #
    fn_out = input("Output data file = ")
    f_out = open(fn_out, "w")
    f_out.write("Input data file = " + fn_in + "\n")

     
    NPerson, NItems, NCat, ID, X = ReadData(f_in, f_out)
    f_out.write('\n\nNPerson = {}'. format(NPerson))
    f_out.write('\nNItems = {}'.format(NItems))
    f_out.write('\nNCat = {}'.format(NCat))

    
    for i in range(len(ID)):
        f_out.write('\n{0}/{1}: '.format((i + 1), ID[i]))
        for v in X[i]:
            f_out.write(' {}'.format(v))

    if (NCat != 3):
        print('\nNumber of Categories is not three !')
        raise Exception('This programm is for items with three categries.')            

    K = NCat
    Ntot = 0
    IDpsn = []
    IDitm = []
    Res = []
    for i in range(NPerson):
        for j in range(NItems):
            if (X[i][j] >= 1) and (X[i][j] <= K):
                IDpsn.append(i + 1)
                IDitm.append(j + 1)
                Res.append(X[i][j])
                Ntot += 1

    f_out.write('\n\nReconstructed data for the Stan script... ')
    for t in range(Ntot):
        f_out.write('\n{0}/{1}  IDpsn/{2}  IDitm/{3}  Res/{4}'.format(
            (t + 1), Ntot, IDpsn[t], IDitm[t], Res[t]))

    coeff_alpha = CalcAlpha(X, NCat, NPerson, NItems)
    if coeff_alpha < 0.0:
        print('\nA missing value was found in the dataset.')
        print('The coefficient alpha was not calculated.')
        f_out.write('\n\nA missing value was found in the dataset.\n')
        f_out.write('The coefficient alpha was not calculated.\n')
    else:
        print('\nCoeff_alpha = {0:.3f}\n'.format(coeff_alpha))
        f_out.write('\n\nCoeff_alpha = {0:.3f}\n'.format(coeff_alpha))

    Data = {'K': NCat, 'Npsn': NPerson, 'Nitm': NItems, 'Ntot': Ntot,
            'IDpsn': IDpsn, 'IDitm': IDitm, 'Res': Res}

    iLmbd = []
    imu = []
    ipsi = []
    for i in range(NItems):
        iLmbd.append(1.0)
        imu.append(0.0)
        ipsi.append(1.0)

    def f_init():
        return dict(Lambda = iLmbd, mu = imu, psi = ipsi)


    
    fit = pystan.stan(file = 'ctt_ThreeCat.stan', data = Data,
                      init = f_init, seed = 999,
                      pars = ['Lambda', 'mu', 'psi', 'C'], n_jobs = 1)
    

    """
    sm = StanModel(file = 'ctt_ThreeCat.stan')
    with open('model.pkl', 'wb') as f:
        pickle.dump(sm, f)

    fit = sm.sampling(data = Data, seed = 999,
                      pars = ['Lambda', 'mu', 'psi', 'C'], n_jobs = 1)
    with open('fit.pkl', 'wb') as g:
        pickle.dump(fit, g)
    """


    """
    sm = pickle.load(open('model.pkl', 'rb'))
    fit = pickle.load(open('fit.pkl', 'rb'))
    """
    

    print(fit)

    f_out.write('\n\nfit...\n')
    f_out.write('{}'.format(fit))

    Llmbd = fit['Lambda']
    Lmu = fit['mu']
    Lpsi = fit['psi']
    LC = fit['C']
    RawNSamples = len(Llmbd)

    rn = RNp2to191()

    LSamplesID = []
    for i in range(RawNSamples):
        LSamplesID.append([])
        LSamplesID[i].append(i)
        LSamplesID[i].append(rn.uni())
    
    LSamplesID.sort(key = lambda v: v[1])

    MCMCSmpl_lmbd = []
    for i in range(NItems):
        MCMCSmpl_lmbd.append([])
    for v in Llmbd:
        for i in range(NItems):
            MCMCSmpl_lmbd[i].append(v[i])

    MCMCSmpl_mu = []
    MCMCSmpl_psi = []
    MCMCSmpl_C = []
    for i in range(NItems):
        MCMCSmpl_mu.append([])
        MCMCSmpl_psi.append([])

    for i in range(NCat - 1):
        MCMCSmpl_C.append([])

    for v in Lmu:
        for i in range(NItems):
            MCMCSmpl_mu[i].append(v[i])
    for v in Lpsi:
        for i in range(NItems):
            MCMCSmpl_psi[i].append(v[i])
    for v in LC:
        for i in range(NCat - 1):  
            MCMCSmpl_C[i].append(v[i])

    for i in range(NItems):
        MCMCSmpl_lmbd[i].sort()
        MCMCSmpl_mu[i].sort()
        MCMCSmpl_psi[i].sort()
   
    for i in range(NCat - 1):
        MCMCSmpl_C[i].sort()

    Med_Lmbd = []
    for i in range(NItems):
        Med_Lmbd.append(MCMCSmpl_lmbd[i][int(len(MCMCSmpl_lmbd[i]) / 2.0)])

    Med_mu = []
    for i in range(NItems):
        Med_mu.append(MCMCSmpl_mu[i][int(len(MCMCSmpl_mu[i]) / 2.0)])
    Med_psi = []
    for i in range(NItems):
        Med_psi.append(MCMCSmpl_psi[i][int(len(MCMCSmpl_psi[i]) / 2.0)])
    Med_C = []
    for i in range(NCat - 1):       
        Med_C.append(MCMCSmpl_C[i][int(len(MCMCSmpl_C[i]) / 2.0)])

    print('\nMedians of Lambdas...')
    for i in range(NItems):
        print("Med_Lambda[{0}] = {1:.3f}".format((i + 1), Med_Lmbd[i]))
    print('\nMedians of mus...')
    for i in range(NItems):
        print('Med_mu[{0}] = {1:.3f}'.format((i + 1), Med_mu[i]))
    print('\nMedians of psis...')
    for i in range(NItems):
        print('Med_psi[{0}] = {1:.3f}'.format((i + 1), Med_psi[i]))
    print('\nMedians of Cs...')
    for i in range(NCat - 1):      
        print('Med_C[{0}] = {1:.3f}'.format((i + 1), Med_C[i]))

   
    f_out.write('\n\nMedian of Lambdas...\n')
    for i in range(NItems):
        f_out.write("Med_Lambda[{0}] = {1:.3f}\n".format((i + 1), Med_Lmbd[i]))
    f_out.write('\nMedians of mus...\n')
    for i in range(NItems):
        f_out.write('Med_mu[{0}] = {1:.3f}\n'.format((i + 1), Med_mu[i]))
    f_out.write('\nMedians of psis...\n')
    for i in range(NItems):
        f_out.write('Med_psi[{0}] = {1:.3f}\n'.format((i + 1), Med_psi[i]))
    f_out.write('\nMedians of Cs...\n')
    for i in range(NCat - 1):      
        f_out.write('Med_C[{0}] = {1:.3f}\n'.format((i + 1), Med_C[i]))

 

    print('\nCalculating Rho_cat...')
    calc_rhos = CalcRhoPreRhoYYP( NCat, NItems, Med_mu, Med_Lmbd, Med_psi, Med_C )
    print('\nEst_Rho_cat by Meds = {0:.3f}'.format(calc_rhos.GetRhoPre()))
    f_out.write('\nEst_Rho_cat by Meds = {0:.3f}\n'.format(calc_rhos.GetRhoPre()))

    print('\nDo you estimate the distribution of Rho_cat?  Y(es) or N(o) = ', end = '')
    ck_est = input()
    if (ck_est == 'Y') or (ck_est == 'y'):
        print('Estimation started.')
        StartPos = 0
        StopPos = 9
        param0 = {'NCat': NCat, 'NItems': NItems, 'StartPos': StartPos, 'StopPos': StopPos,
                   'LSamplesID': LSamplesID, 'Llmbd': Llmbd, 'Lmu': Lmu, 'Lpsi': Lpsi, 'LC': LC}
        n_proc = 4;
        params = []
        NEstRhos = None
        if (RawNSamples > 100):
            NEstRhos = 100
        else:
            NEstRhos = RawNSamples
            
        for iproc in range(n_proc):
            param_t = param0.copy()
            param_t['StartPos'] = int(iproc * NEstRhos / n_proc)
            param_t['StopPos'] = int(((iproc + 1) * NEstRhos / n_proc) - 1)
            params.append(param_t)

        pool = Pool()
        Results = pool.map(EstDistriRho_cat, params)

        SamplesRho_cat = []
        for u in Results:
            for v in u:
                SamplesRho_cat.append(v)

        sum = 0.0
        for v in SamplesRho_cat:
            sum += v
        meanRho_cat = sum / len(SamplesRho_cat)
        SamplesRho_cat.sort()

        medRho_cat = SamplesRho_cat[int(len(SamplesRho_cat) / 2)]
        Q1Rho_cat = SamplesRho_cat[int(len(SamplesRho_cat) / 4)]
        Q3Rho_cat = SamplesRho_cat[int(len(SamplesRho_cat) * 3 / 4)]

        print('\nMean of Rho_cat = {0:.3f}'.format(meanRho_cat))
        print('Median of Rho_cat = {0:.3f}'.format(medRho_cat))
        print('[Q1, Q3] = [{0:.3f}, {1:.3f}]'.format(Q1Rho_cat, Q3Rho_cat))

        f_out.write('\n\nMean of Rho_cat = {0:.3f}\n'.format(meanRho_cat))
        f_out.write('Median of Rho_cat = {0:.3f}\n'.format(medRho_cat))
        f_out.write('[Q1, Q3] = [{0:.3f}, {1:.3f}]\n'.format(Q1Rho_cat, Q3Rho_cat))
        
        title_str = 'Mean of Rho_cat = {0:.3f}\n' + \
                        '(Q1, Med, Q3) = ({1:.3f}, {2:.3f}, {3:.3f})'
        title_str = title_str.format(meanRho_cat, Q1Rho_cat, medRho_cat, Q3Rho_cat)
        plt.title(title_str)
        plt.hist(SamplesRho_cat, bins = 10)
        plt.show()


    f_out.close()
     
