//      Yasuharu Okamoto, 2017.07

data {
    int <lower = 3, upper = 3> K;                          //      Number of categories = 3
    int Npsn;                                   //      Number of persons
    int Nitm;                                   //      Number of items
    int Ntot;                                   //      Number of data,  Ntot <= Nprn * Nitm
    int<lower = 1, upper = Npsn> IDpsn[Ntot];   //      Person ID
    int<lower = 1, upper = Nitm> IDitm[Ntot];   //      Item ID
    int<lower = 1, upper = K> Res[Ntot];        //      Response-> An integer value between 1 and K
}

parameters {
        real<lower = 0.0> Lambda[Nitm];
        real mu[Nitm];
        real<lower = 0.0> psi[Nitm];
        real F[Npsn];
}

transformed parameters {
    vector[K] p[Ntot];
    real C[K - 1];

    C[1] = -1.0;
    C[2] = 1.0;

    for (i in 1:Ntot){
        p[i][K] = Phi(((mu[IDitm[i]] + (Lambda[IDitm[i]] * F[IDpsn[i]])) - C[K - 1]) /
                      psi[IDitm[i]]);
        p[i][1] =1.0 - Phi(((mu[IDitm[i]] + (Lambda[IDitm[i]] * F[IDpsn[i]])) - C[1]) /
                      psi[IDitm[i]]);
        
        for (k in 2:(K - 1)){
            p[i][k] = Phi(((mu[IDitm[i]] + (Lambda[IDitm[i]] * F[IDpsn[i]])) - C[k - 1]) /
                      psi[IDitm[i]])
                      - Phi(((mu[IDitm[i]] + (Lambda[IDitm[i]] * F[IDpsn[i]])) - C[k]) /
                      psi[IDitm[i]]);
        }
    }
}

model {
    for (i in 1:Npsn)
        F[i] ~ normal(0.0, 1.0);
    for (i in 1:Ntot)
        Res[i] ~ categorical(p[i]);
}
